/*
 * File:   graph.cc
 * Author: Filip Niksic, fniksic@gmail.com
 *
 * Created on 2008. sijecanj 31, 00:07
 */

#include <iostream>
#include <limits>
#include <iterator>
#include <algorithm>
#include <utility>
#include <set>

#include "graph.h"

using namespace std;

const Graph_data::Vertex_ID Graph_data::nullVertex = numeric_limits<Graph_data::Vertex_ID>::max();
const Graph_data::Edge_ID Graph_data::nullEdge = make_pair(nullVertex, nullVertex);

const Graph::Vertex_ID Graph::nullVertex = Graph_data::nullVertex;
const Graph::Edge_ID Graph::nullEdge = Graph_data::nullEdge;

void Graph_data::clearEdges() {
    for (Matrix::iterator it = adjMatrix.begin(); it != adjMatrix.end(); ++it)
        for (Row::iterator jt = it->begin(); jt != it->end(); ++jt)
            *jt = Inf;
}

Graph_data::Vertex_ID Graph_data::addVertex() {
    ++size;
    for (Matrix::iterator it = adjMatrix.begin(); it != adjMatrix.end(); ++it)
        it->push_back(Inf);
    adjMatrix.push_back(Row(size, Inf));
    return size - 1;
}

void Graph_data::delVertex(Vertex_ID v) {
    validateVertex(v);
    // Brisem v-ti redak matrice
    adjMatrix.erase(adjMatrix.begin() + v);
    // Brisem v-ti stupac matrice
    for (Matrix::iterator it = adjMatrix.begin(); it != adjMatrix.end(); ++it)
        it->erase(it->begin() + v);
}

void Graph_data::print(ostream &os) const {
    ostream_iterator<Real_T> out(os, "\t");
    for (Matrix::const_iterator it = adjMatrix.begin(); it != adjMatrix.end(); ++it) {
        copy(it->begin(), it->end(), out);
        os << endl;
    }
}

Graph_data* Graph_data::subGraph(const set<Vertex_ID> &subset) const {
    Graph_data *newGraph = new Graph_data(subset.size());
    set<Vertex_ID>::const_iterator it, jt;
    Vertex_ID i, j;
    for (i = 0, it = subset.begin(); it != subset.end(); ++i, ++it)
        for (j = i, jt = it; jt != subset.end(); ++j, ++jt)
            newGraph->setEdge(i, j, getEdge(*it, *jt));
    return newGraph;
}

ostream &operator<<(ostream &os, const Graph &g) {
    g.ptr->print(os);
    return os;
}
