#include <iostream>
#include <fstream>
#include <sstream>
#include <cstddef>
#include <cstdlib>

#include "globals.h"
#include "graph.h"
#include "tspsolver.h"

using namespace std;

int main(const int argc, const char * const argv[]) {
    int exitStatus = EXIT_SUCCESS;

    if (argc < 3) {
	cout << "Usage: " << argv[0] << " <problem_size> <input_file>" << endl;
	exitStatus = EXIT_FAILURE;
	return exitStatus;
    }

    istringstream sStream(argv[1]);
    size_t size;
    sStream >> size;

    Graph g(size);
    ifstream inFile(argv[2]);

    Real_T tmp;
    for (Graph::Vertex_ID u = 0; u < size; ++u)
	for (Graph::Vertex_ID v = 0; v <= u; ++v) {
	    inFile >> tmp;
	    if (u != v) g.setEdge(u, v, tmp);
	}
    inFile.close();

    TSPSolver tsp(g);
    tsp.solve();
    if (tsp.tourFound())
	cout << "Best tour cost: " << tsp.tourCost() << endl;
    else
	cout << "Tour not found." << endl;

    return exitStatus;
}
