/*
 * File:   tspsolver.h
 * Author: Filip Niksic, fniksic@gmail.com
 *
 * Created on 2008. veljaca 08, 01:04
 */

#ifndef TSPSOLVER_H
#define	TSPSOLVER_H

#include <vector>
#include <stdexcept>
#include <iostream>

#include "globals.h"
#include "graph.h"
#include "branch.h"

class TSPSolver {
    Graph g;
    Branch *best;
    Real_T bestTour; // dvostruka duljina ture
public:
    explicit TSPSolver(const Graph& _g) : g(_g), best(0), bestTour(Inf) {}
    ~TSPSolver() { delete best; }
    void solve() {
        Branch *root = new Branch(g);
        branchAndBound(root);
        delete root;
    }
    bool tourFound() const {
        return best != 0;
    }
    std::vector<Graph::Vertex_ID> getTour() const {
        if (best == 0)
            throw std::runtime_error("TSPSolver::getTour : Tour not found!");
        return best->getTour();
    }
    Real_T tourCost() const {
        if (best == 0)
            throw std::runtime_error("TSPSolver::getTour : Tour not found!");
        return bestTour / 2.0;
    }
private:
    void branchAndBound(Branch*);
};

#endif	/* TSPSOLVER_H */

